﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using Sailboat.DrawingObjects;
using Sailboat.Generics;
using Sailboat.MovementStrategy;
using Sailboat.Entities;

namespace Sailboat
{
    public partial class FormBoatConfig : Form
    {
        DrawingBoat? _boat = null;
        private event Action <DrawingBoat>? EventAddBoat;
        public FormBoatConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;

            buttonCancel.Click += (s, e) => Close();
        }

        private void DrawBoat()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _boat?.SetPosition(5, 5);
            _boat?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        internal void AddEvent(Action<DrawingBoat> ev)
        {
            if (EventAddBoat == null)
            {
                EventAddBoat = ev;
            }
            else
            {
                EventAddBoat += ev;
            }
        }
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name,
           DragDropEffects.Move | DragDropEffects.Copy);
        }
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _boat = new DrawingBoat((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _boat = new DrawingSailboat((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxHull.Checked,
                   checkBoxSail.Checked, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
            }
            DrawBoat();
        }

        private void LabelColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_boat == null)
                return;
            switch (((Label)sender).Name)
            {
                case "labelColor":
                    _boat.EntityBoat.setBodyColor((Color)e.Data.GetData(typeof(Color)));
                    break;
                case "labelAddColor":
                    if (!(_boat is DrawingSailboat))
                        return;
                    (_boat.EntityBoat as EntitySailboat).setAdditionalColor((Color)e.Data.GetData(typeof(Color)));
                    break;
            }
            DrawBoat();
        }
        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddBoat?.Invoke(_boat);
            Close();
        }
    }
}
